# LiT-4-RSVQA: Lightweight Transformer-based Visual Question Answering in Remote Sensing
This repository contains code of the paper [`LiT-4-RSVQA: Lightweight Transformer-based 
Visual Question Answering in Remote Sensing`](https://arxiv.org/abs/2306.00758) presented at the IEEE International Geoscience and Remote Sensing Symposium (IGARSS) in July 2023. This work has been done at the [Remote Sensing Image Analysis group](https://rsim.berlin/) by [Leonard Hackel](https://rsim.berlin/team/members/leonard-hackel), [Kai Norman Clasen](https://rsim.berlin/team/members/kai-norman-clasen), [Mahdyar Ravanbakhsh](https://rsim.berlin/team/alumni) and [Begüm Demir](https://rsim.berlin/team/members/begum-demir).

If you use this code, please cite our paper given below:

> L. Hackel and K. N. Clasen and M. Ravanbakhsh and B. Demіr, "LiT-4-RSVQA: Lightweight Transformer-based Visual Question Answering in Remote Sensing", IEEE International Geoscience and Remote Sensing Symposium, Pasadena, California, 2023.

```bibtex
@INPROCEEDINGS{10281674,
    author={Hackel, Leonard and Clasen, Kai Norman and Ravanbakhsh, Mahdyar and Demir, Begüm},
    booktitle={IGARSS 2023 - 2023 IEEE International Geoscience and Remote Sensing Symposium}, 
    title={LIT-4-RSVQA: Lightweight Transformer-Based Visual Question Answering in Remote Sensing}, 
    year={2023},
    volume={},
    number={},
    pages={2231-2234},
    doi={10.1109/IGARSS52108.2023.10281674}
}
```

## Introduction
Visual question answering (VQA) methods in remote sensing (RS) aim to answer natural language questions with respect to an RS image.
Most of the existing methods require a large amount of computational resources, which limits their application in operational scenarios in RS.
To address this issue, in this paper we present an effective lightweight transformer-based VQA in RS (LiT-4-RSVQA) architecture for efficient and accurate VQA in RS.
Our architecture consists of: 
i) a lightweight text encoder module; 
ii) a lightweight image encoder module; 
iii) a fusion module; 
and iv) a classification module.
The experimental results obtained on a VQA benchmark dataset demonstrate that our proposed LiT-4-RSVQA architecture provides accurate VQA results while significantly reducing the computational requirements on the executing hardware.

## Dataset
Two datasets were used for training:
1. Pre-training: [BigEarthNet](https://bigearth.net/) (only Sentinel-2)
2. Training: [RSVQAxBEN](https://zenodo.org/record/5084904) extended with all available Sentinel-2 10m and 20m bands included in [BigEarthNet](https://bigearth.net/) 

## Prerequisites
The code in this repository uses the requirements specified in `conda_env.yml`. To install the requirements, call `conda env create -f conda_env.yml`.

The code is tested in Ubuntu 22.04 on NVidia A100 GPUs with Driver Version 515.105.01 (CUDA Version: 11.7)

Create an account at [Weights & Biases](https://wandb.ai/) and save the `API KEY` in an environment variable named `WANDB_API_KEY` (e.g. `export WANDB_API_KEY=abcdefg` in the executing shell) for logging of results.  

## Pre-Training
To pretrain the image encoder, call
```
python pretrain_lit4rsvqa.py --batch-size 512 --epochs 100 --model-name <MODEL_TO_TRAIN> --data-dir <path/to/BigEarthNet/Encoder/BigEarthNetEncoded.lmdb>
```
Optionally the following parameters can be set:  
`--lr FLOAT` the learning rate  
`--num-workers-dataloader INT` number of workers used in pytorch dataloader  
`--seed INT` seed for initialization of RNG  
`--test-run` or `--no-test-run` (default) only use limited number of batches to see if different steps of training work
see also: `python pretrain_lit4rsvqa.py --help`

The training will validate every 5th epoch and save a checkpoint with the best F1 score.
After training/validation, the best checkpoint will be tested.
Training progress and results are saved locally and on [Weights & Biases](https://wandb.ai/).

For the publication, the seed 42 was used.

## Training and Evaluation

To train the vqa encoder end-to-end, call
```
python train_lit4rsvqa.py --vision-model <VISION_MODEL_TO_TRAIN> --data-dir <path/to/TRAININGFILES>
```
or with pre-trained vision encoder
```
python train_lit4rsvqa.py --vision-model <VISION_MODEL_TO_TRAIN> --vision-checkpoint <path/to/vision/weights> --data-dir <path/to/TRAININGFILES>
```
where the weights are pytorch weights or a lightning checkpoint including the weights.
The `TRAININGFILES` should be set up as follows:
```
<TRAININGFILES>
├── BigEarthNetEncoded.lmdb
│   ├── data.mdb
│   └── lock.mdb
└── VQA_RSVQAxBEN
    ├── RSVQAxBEN_QA_test.json
    ├── RSVQAxBEN_QA_test_subset.json
    ├── RSVQAxBEN_QA_train.json
    ├── RSVQAxBEN_QA_train_subset.json
    ├── RSVQAxBEN_QA_val.json
    └── RSVQAxBEN_QA_val_subset.json
```

Optionally the following parameters can be set:  
`--lr FLOAT` the learning rate  
`--num-workers-dataloader INT` number of workers used in pytorch dataloader  
`--seed INT` seed for initialization of RNG  
`--test-run` or `--no-test-run` (default) only use limited number of batches to see if different steps of training work  
`--text-model STR` the learning rate  
`--epochs INT` number of epochs - this changes the LR schedule automatically  
`--batch-size INT` the batch size  
`--matmul-precision STR` precision used in [torch matmul](https://pytorch.org/docs/stable/generated/torch.set_float32_matmul_precision.html)  
see also: `python train_lit4rsvqa.py --help`

For the publication, the seeds 42, 1337 and 2023 were used. 
Results can be found in [this file](results.csv).

## Authors
**Leonard Hackel**
https://rsim.berlin/team/members/leonard-hackel

**Kai Norman Clasen**
https://rsim.berlin/team/members/kai-norman-clasen 

**Mahdyar Ravanbakhsh**
https://rsim.berlin/team/alumni

**Begüm Demir**
https://rsim.berlin/team/members/begum-demir

For questions, requests and concerns, please contact [Leonard Hackel via mail](mailto:l.hackel@tu-berlin.de)

## License
The code in this repository is licensed under the **MIT License**:
```
MIT License

Copyright (c) 2023 Leonard Hackel

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
```

